   /**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */
#import "RouteViewController.h"
#import <TomTomOnlineSDKMapsUIExtensions/TomTomOnlineSDKMapsUIExtensions.h>
#import <TomTomOnlineSDKMaps/TomTomOnlineSDKMaps.h>
#import "Locations.h"
#import "OverlayDrawing.h"
#import "ETAView.h"
#import "ProgressUtil.h"
#import "Colors.h"

typedef NS_ENUM(NSInteger, ConsumptionMode) {
    ConsumptionModeFuel,
    ConsumptionModeBattery
};

@interface RouteViewController () < TTRouteResponseDelegate , UIExampleViewControllerDelegate, TTAnnotationDelegate, TTMapViewDelegate, TTRouteDelegate, TTReachableRangeDelegate>

@property (nonatomic, weak) IBOutlet TTMapView *mapView;
@property (weak, nonatomic) IBOutlet TTControlView *controlView;
@property (weak, nonatomic) IBOutlet ETAView *etaView;
@property (nonatomic, assign) RouteMode mode;
@property (nonatomic, assign) TTMapTilesType mapTiles;
@property (nonatomic, strong) ProgressUtil *progress;
@property TTReachableRange *reachableRange;
@property TTRoute* routePlanner;
@property TTRouteQuery *query;
@property CLLocationCoordinate2D *coordinates;
@property (nonatomic, strong) TTAnnotation *dest;
@property (nonatomic, strong) TTAnnotation *orygin;
@property NSInteger index;
@property UIDatePicker *datePicker;
@property NSDictionary *polylineIDArray;
@property NSNumber* selectedRoute;
@property NSArray *colorArray;
@property ConsumptionMode consumptionMode;

@end

@implementation RouteViewController

- (void)reachableRange:(TTReachableRange *)range completedWithResult:(TTReachableRangeResponse *)response {
    [self.progress cancelProgress];
    [self.mapView.annotationManager removeAllOverlays];
    
    CLLocationCoordinate2D boundries[response.result.boundriesCount];
    for (int i = 0; i < response.result.boundriesCount; i++) {
        CLLocationCoordinate2D boundry = [response.result boundryAt:i];
        boundries[i] = boundry;
    }
    
    TTPolygon *polygon = [TTPolygon polygonWithCoordinates:boundries count:response.result.boundriesCount opacity:1 color:TTColorOverlay colorOutline:TTColorOverlayOutline];
    [self.mapView.annotationManager addOverlay:polygon];
    
    [self.mapView zoomToCoordinatesData:polygon];
}

- (void)reachableRange:(TTReachableRange *)range completedWithResponseError:(TTResponseError *)responseError {
    [self setShowToastEnable:true];
    [self showToast:[NSString stringWithFormat:@"Error: %@", [responseError.userInfo objectForKey:@"description"]] withTimer:NO];
    [self.progress cancelProgress];
    [self deselectOptionIndexs:@[@(3), @(2), @(1)]];
}

- (void)route:(TTRoute *)route completedWithResult:(TTRouteResult *)result{
    [self.progress cancelProgress];
    [self.mapView.routeManager removeAllRoutes];
    
    BOOL markAsActive = YES;
    for(TTFullRoute *fullRoute in result.routes) {
        TTMapRoute *mapRoute = [TTMapRoute routeWithCoordinatesData:fullRoute imageStart:[UIImage imageNamed:@"ic_map_route_departure"] imageEnd:[UIImage imageNamed:@"ic_map_route_destination"]];
        mapRoute.extraData = fullRoute.summary;
        [self.mapView.routeManager addRoute:mapRoute];
        if(markAsActive){
            [self updateETA:mapRoute];
            mapRoute.active = YES;
            markAsActive = NO;
        }
    }
    [self displayRouteOverview];
}

- (void)route:(TTRoute *)route completedWithResponseError:(TTResponseError *)resoultError{
    [self.mapView.annotationManager removeAllOverlays];
    [self setShowToastEnable:true];
    [self showToast:[NSString stringWithFormat:@"Error: %@", [resoultError.userInfo objectForKey:@"description"]] withTimer:NO];
    [self.progress cancelProgress];
    [self deselectOptionIndexs:@[@(2), @(1)]];
}

- (void)displayRouteOverview {
    UIEdgeInsets insets = UIEdgeInsetsMake(self.etaView.frame.origin.y + self.etaView.frame.size.height + 15 * UIScreen.mainScreen.scale,
                                           10 * UIScreen.mainScreen.scale,
                                           self.mapView.frame.size.height - self.optionsContainer.frame.origin.y + 10 * UIScreen.mainScreen.scale,
                                           10 * UIScreen.mainScreen.scale);
    self.mapView.contentInset = insets;
    [self.mapView.routeManager showAllRoutesOverview];
}

- (void)updateETA:(TTMapRoute *)mapRoute {
    TTSummary *summary = (TTSummary *)mapRoute.extraData;
    NSString *dateInString = [NSString alloc];
    if(_mode == RouteConsumptionModel){
        switch (self.consumptionMode) {
            case ConsumptionModeBattery:
            
                dateInString = [NSString stringWithFormat:@"Consumption: %.2f kWh",[summary.batteryConsumptionInkWh doubleValue]];
                break;
            case ConsumptionModeFuel:
                
                dateInString = [NSString stringWithFormat:@"Consumption: %.2f liters",[summary.fuelConsumptionInLiters doubleValue]];
                break;
        }
    }else{
        NSDate *eta = self.mode == RouteTimeTravel && self.index == 1 ? summary.departureTime : summary.arrivalTime;
        NSCalendar *calendar = [NSCalendar currentCalendar];
        [calendar setTimeZone: [NSTimeZone localTimeZone]];
        NSDateComponents *components = [calendar components:(NSCalendarUnitHour | NSCalendarUnitMinute) fromDate:eta];
        dateInString = [NSString stringWithFormat:@"%02ld:%02ld", components.hour, components.minute];
    }
    [self.etaView updateETA:dateInString kmToDistance:summary.lengthInMeters.integerValue / 1000];
    [self showETA];
}

- (void)viewDidLayoutSubviews {
    [super viewDidLayoutSubviews];
    CGRect frame = self.etaView.frame;
    frame.origin.y = self.navigationController.navigationBar.frame.origin.y
    + self.navigationController.navigationBar.frame.size.height;
    self.etaView.frame = frame;
}

- (void)showETA {
    self.etaView.hidden = NO;
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass * 2;
}

- (void)hideETA {
    self.etaView.hidden = YES;
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    _colorArray = [[NSArray alloc] initWithObjects:[UIColor redColor],[UIColor yellowColor], TTColorLightStone,[UIColor greenColor],TTColorMidgrey,TTColorLightStone,TTColorLight,nil];
    self.controlView.mapView = self.mapView;
    [self.controlView initDefaultCenterButton];
    [self.controlView initDefaultCompassButton];
    self.controlView.topLayoutConstraintCompassButton.constant = TopLayoutConstraintCompass;
    self.controlView.bottomLayoutConstraintCenterButton.constant = BottomLayoutConstraintCenter;
    
    _routePlanner = [[TTRoute alloc] init];
    _routePlanner.delegate = self;
    
    _reachableRange = [[TTReachableRange alloc] init];
    _reachableRange.delegate = self;
    
    self.delegate = self;
    self.mapView.annotationManager.delegate = self;
    self.mapView.delegate = self;
    self.mapView.showsUserLocation = YES;
    self.mapView.routeManager.delegate = self;
    
    [self hideETA];
    self.progress = [[ProgressUtil alloc] initWithViewController:self title:@"Routing..." subtitle:nil];
    [self.mapView centerOnCoordinate:COORDINATE_AMSTERDAM withZoom:10];
    switch (self.mode) {
        case TravelModes:
            [self buildHeader:@"Travel modes" and:@"Amsterdam to Rotterdam"];
            [self createSegmentedControllWithTag:TravelModes options:@[@"Car",@"Truck", @"Pedestrian"] multiSelect:NO select:4];
            break;
            
        case RouteTypes:
            [self buildHeader:@"Route types" and:@"Amsterdam to Rotterdam"];
            [self createSegmentedControllWithTag:RouteTypes options:@[@"Fastest", @"Shortest", @"Eco"] multiSelect:NO select:4];
            break;
            
        case RouteAvoids:
            [self buildHeader:@"Route avoids" and:@"Amsterdam to Oslo"];
            [self createSegmentedControllWithTag:RouteAvoids options:@[@"Motorways", @"Toll roads", @"Ferries"] multiSelect:NO select:4];
            break;
        case RouteWithWaypoints:
            [self buildHeader:@"Route with waypoints" and:@"Amsterdam to Berlin"];
            [self createSegmentedControllWithTag:RouteWithWaypoints options:@[@"Initial order", @"Best Order", @"No waypoints"] multiSelect:NO select:4];
            break;
        case RouteTimeTravel:
            [self buildHeader:@"Departure/arrival time" and:@"Amsterdam to Rotterdam"];
            [self createSegmentedControllWithTag:RouteTimeTravel options:@[@"Departure at", @"Arrival at"] multiSelect:NO select:3];
            break;
        case RouteAlternative:
            [self buildHeader:@"Alternative routes" and:@"Amsterdam to Rotterdam"];
            [self createSegmentedControllWithTag:RouteAlternative options:@[@"1", @"3", @"5"] multiSelect:NO select:4];
            break;
        case RouteConsumptionModel:
            [self buildHeader:@"Consumption model" and:@"Amsterdam to Utrecht"];
            [self createSegmentedControllWithTag:RouteConsumptionModel options:@[@"Combustion", @"Electric"] multiSelect:NO select:4];
            break;
        case RouteSupportingPoints:
            [self buildHeader:@"Supporting points" and:@"Choose min. deviation distanse"];
            [self createSegmentedControllWithTag:RouteSupportingPoints options:@[@"0 m", @"10 km"] multiSelect:NO select:4];
            break;
        case ReachableRange:
            [self buildHeader:@"Reachable range" and:@"Amsterdam"];
            [self createSegmentedControllWithTag:ReachableRange options:@[@"Combustion", @"Electric", @"Time - 2h"] multiSelect:NO select:4];
            break;
    }
    
}
- (void)setRouteMode:(RouteMode)mode {
    self.mode = mode;
}

- (void)resultTravelModes:(NSInteger)index {
    TTOptionTravelMode travelMode = TTOptionTravelModeNone;
    switch (index) {
        case 0:
            travelMode = TTOptionTravelModeCar;
            break;
        case 1:
            travelMode = TTOptionTravelModeTruck;
            break;
        case 2:
            travelMode = TTOptionTravelModePedestrian;
            break;
    }
    _query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_ROTTERDAM andOrig:COORDINATE_AMSTERDAM]
               withTravelMode:travelMode]
               withTraffic:NO]
              build];
    
    
    [_routePlanner planRouteWithQuery:_query];
    
    [self displayProgressDialog];
}

- (void)resultRouteTypes:(NSInteger)index {
    TTOptionTypeRoute routeType = TTOptionTypeRouteNone;
    switch (index) {
        case 0:
            routeType = TTOptionTypeRouteFastest;
            break;
        case 1:
            routeType = TTOptionTypeRouteShortest;
            break;
        case 2:
            routeType = TTOptionTypeRouteEco;
            break;
    }
    
    _query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_ROTTERDAM andOrig:COORDINATE_AMSTERDAM]
               withRouteType:routeType]withTraffic:NO]
              build];
    
    [_routePlanner planRouteWithQuery:_query];
    
    
    [self displayProgressDialog];
}

- (void)resultRouteAvoids:(NSInteger)index {
    [self.mapView centerOnCoordinate:CLLocationCoordinate2DMake(54.929440, 6.464524) withZoom:4];
    TTOptionTypeAvoid avoidType = TTOptionTypeAvoidNone;
    switch (index) {
        case 0:
            avoidType = TTOptionTypeAvoidMotorways;
            break;
        case 1:
            avoidType = TTOptionTypeAvoidTollRoads;
            break;
        case 2:
            avoidType = TTOptionTypeAvoidFerries;
            break;
    }
    
    _query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_OSLO andOrig:COORDINATE_AMSTERDAM]
               withAvoidType:avoidType]withTraffic:NO]
              build];

    [_routePlanner planRouteWithQuery:_query];

    
    [self displayProgressDialog];
}

- (void)addAnnotationsForCoordinate:(CLLocationCoordinate2D)coordinate {
    TTAnnotation *annotation = [TTAnnotation annotationWithCoordinate:coordinate];
    [self.mapView.annotationManager addAnnotation:annotation];
}

- (void)resoultRouteWaypoints:(NSInteger)index {
    CLLocationCoordinate2D brussels = COORDINATE_BRUSSELS;
    CLLocationCoordinate2D hamburg = COORDINATE_HAMBURG;
    CLLocationCoordinate2D zurich = COORDINATE_ZURICH;
    NSArray* waypoints = @[
                           [NSValue value:&hamburg withObjCType:@encode(CLLocationCoordinate2D)],
                           [NSValue value:&zurich withObjCType:@encode(CLLocationCoordinate2D)],
                           [NSValue value:&brussels withObjCType:@encode(CLLocationCoordinate2D)]
                           ];
   
    [self.mapView.annotationManager removeAllAnnotations];
    [self.mapView.routeManager removeAllRoutes];
    switch (index) {
        case 0:
            _query = [[[[[TTRouteQueryBuilder createWithDest:COORDINATE_BERLIN andOrig:COORDINATE_AMSTERDAM]
                                                    withWayPoints:waypoints]
                                             withComputeBestOrder:NO] withTraffic:NO]build];
            
            [_routePlanner planRouteWithQuery:_query];
            
            [self addAnnotationsForCoordinate:brussels];
            [self addAnnotationsForCoordinate:hamburg];
            [self addAnnotationsForCoordinate:zurich];
            break;
        case 1:
            _query = [[[[[TTRouteQueryBuilder createWithDest:COORDINATE_BERLIN andOrig:COORDINATE_AMSTERDAM]
                                                    withWayPoints:waypoints]
                                             withComputeBestOrder:YES]withTraffic:NO] build];
            
            [_routePlanner planRouteWithQuery:_query];
    
            
            [self addAnnotationsForCoordinate:brussels];
            [self addAnnotationsForCoordinate:hamburg];
            [self addAnnotationsForCoordinate:zurich];
            break;
        case 2:
            _query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_BERLIN andOrig:COORDINATE_AMSTERDAM]
                       withComputeBestOrder:YES] withTraffic:NO]build];
            [_routePlanner planRouteWithQuery:_query];
    
            break;
        default:
            break;
    }
    
    [self displayProgressDialog];
}

- (void)displayProgressDialog {
    [self.progress displayProgressWithAction:^{
        [self.routePlanner cancel];
        [self.mapView.annotationManager removeAllOverlays];
        [self.mapView.annotationManager removeAllAnnotations];
        [self hideETA];
    }];
}

- (void)optionButtonTag:(NSInteger)tag index:(int)index {
    
    switch (tag) {
        case TravelModes:
            self.mode = TravelModes;
            [self resultTravelModes:index];
            break;
        case RouteTypes:
            self.mode = RouteTypes;
            [self resultRouteTypes:index];
            break;
        case RouteAvoids:
            self.mode = RouteAvoids;
            [self resultRouteAvoids:index];
            break;
        case RouteWithWaypoints:
            _index = index;
            self.mode = RouteWithWaypoints;
            [self resoultRouteWaypoints:index];
            break;
        case RouteTimeTravel:
            self.mode = RouteTimeTravel;
            [self resultRouteTimeTravel:index];
            break;
        case RouteAlternative:
            self.mode = RouteAlternative;
            [self resultAlternativeRoute:index];
            break;
        case RouteConsumptionModel:
            self.mode = RouteConsumptionModel;
            [self resultConsumptionModel:index];
            break;
        case RouteSupportingPoints:
            self.mode = RouteSupportingPoints;
            [self resultSupportingPoints:index];
            break;
        case ReachableRange:
            self.mode = ReachableRange;
            [self resultReachableRange:index];
            break;
        default:
            break;
    }
}

- (void)resultConsumptionModel:(NSInteger)index {

    TTSpeedConsumption speedConsumptionInkWh[6];
    speedConsumptionInkWh[0] = TTSpeedConsumptionMake(10, 5.0);
    speedConsumptionInkWh[1] = TTSpeedConsumptionMake(30, 10.0);
    speedConsumptionInkWh[2] = TTSpeedConsumptionMake(50, 15.0);
    speedConsumptionInkWh[3] = TTSpeedConsumptionMake(70, 20.0);
    speedConsumptionInkWh[4] = TTSpeedConsumptionMake(90, 25.0);
    speedConsumptionInkWh[5] = TTSpeedConsumptionMake(120, 30.0);

    TTSpeedConsumption speedConsumptionInLiters[7];
    speedConsumptionInLiters[0] = TTSpeedConsumptionMake(10, 6.5);
    speedConsumptionInLiters[1] = TTSpeedConsumptionMake(30, 7.0);
    speedConsumptionInLiters[2] = TTSpeedConsumptionMake(50, 8.0);
    speedConsumptionInLiters[3] = TTSpeedConsumptionMake(70, 8.4);
    speedConsumptionInLiters[4] = TTSpeedConsumptionMake(90, 7.7);
    speedConsumptionInLiters[5] = TTSpeedConsumptionMake(120, 7.5);
    speedConsumptionInLiters[6] = TTSpeedConsumptionMake(150, 9.0);
    

    [_etaView isIconHidden:YES];
    switch (index) {
        case 0:
            self.consumptionMode = ConsumptionModeFuel;
            _query = [[[[[[[[[[[[[[TTRouteQueryBuilder createWithDest:COORDINATE_UTRECHT andOrig:COORDINATE_AMSTERDAM]
                                 withMaxAlternatives:2]
                                withVehicleWeight:1600] //vehicle weight in kilograms
                               withCurrentFuelInLiters:50.0]
                              withCurrentAuxiliaryPowerInLitersPerHour:0.2]
                             withFuelEnergyDensityInMJoulesPerLiter:34.2]
                            withAccelerationEfficiency:0.33] //e.g. KineticEnergyGained/ChemicalEnergyConsumed
                           withDecelerationEfficiency:0.33] //e.g. ChemicalEnergySaved/KineticEnergyLost
                          withUphillEfficiency:0.33]  //e.g. PotentialEnergyGained/ChemicalEnergyConsumed
                         withDownhillEfficiency:0.33] //e.g. ChemicalEnergySaved/PotentialEnergyLost
                        withVehicleEngineType:TTOptionVehicleEngineTypeCombustion]
                       withSpeedConsumptionInLitersPairs:speedConsumptionInLiters count:7]withTraffic:NO] build];
            break;
        case 1:
            self.consumptionMode = ConsumptionModeBattery;
            _query = [[[[[[[[[[[[[[TTRouteQueryBuilder createWithDest:COORDINATE_UTRECHT andOrig:COORDINATE_AMSTERDAM]
                                withMaxAlternatives:2]
                               withVehicleWeight:1600] //vehicle weight in kilograms
                              withCurrentChargeInkWh:43]
                             withMaxChargeInkWh:85]
                            withAuxiliaryPowerInkW:1.7]
                           withAccelerationEfficiency:0.33] //e.g. KineticEnergyGained/ChemicalEnergyConsumed
                          withDecelerationEfficiency:0.33] //e.g. ChemicalEnergySaved/KineticEnergyLost
                         withUphillEfficiency:0.33] //e.g. PotentialEnergyGained/ChemicalEnergyConsumed
                        withDownhillEfficiency:0.33]  //e.g. ChemicalEnergySaved/PotentialEnergyLost
                        withVehicleEngineType:(TTOptionVehicleEngineTypeElectric)]
                       withSpeedConsumptionInkWhPairs:speedConsumptionInkWh count:6]withTraffic:NO] build];
            break;
    }
    [self displayProgressDialog];
    [_routePlanner planRouteWithQuery:_query];
}

- (void)resultReachableRange:(NSInteger)index {
    TTReachableRangeQuery *query;
    
    TTSpeedConsumption consumption[1];
    consumption[0] = TTSpeedConsumptionMake(50, 6.3);
    
    switch (index) {
        case 0:
            query = [[[[[[[[[[[[[TTReachableRangeQueryBuilder createWithCenterLocation:COORDINATE_AMSTERDAM] withSpeedConsumptionInLitersPairs:consumption count:1]
                             withVehicleWeight:1600]
                            withCurrentFuelInLiters:43]
                           withFuelEnergyDensityInMJoulesPerLiter:34.2]
                          withCurrentAuxiliaryPowerInLitersPerHour:1.7]
                         withAccelerationEfficiency:0.33]
                        withDecelerationEfficiency:0.33]
                       withUphillEfficiency:0.33]
                      withDownhillEfficiency:0.33]
                     withVehicleEngineType:TTOptionVehicleEngineTypeCombustion]
                    withFuelBudgetInLiters:5]
                build];
            break;
        
        case 1:
            query = [[[[[[[[[[[[[TTReachableRangeQueryBuilder createWithCenterLocation:COORDINATE_AMSTERDAM] withSpeedConsumptionInkWhPairs:consumption count:1]
                             withVehicleWeight:1600]
                            withCurrentChargeInkWh:43]
                           withMaxChargeInkWh:85]
                          withAuxiliaryPowerInkW:1.7]
                         withAccelerationEfficiency:0.33]
                        withDecelerationEfficiency:0.33]
                       withUphillEfficiency:0.33]
                      withDownhillEfficiency:0.33]
                     withVehicleEngineType:TTOptionVehicleEngineTypeElectric]
                    withEnergyBudgetInKWh:5]
                build];
            break;
            
        case 2:
            query = [[[[[[[[[[[[[TTReachableRangeQueryBuilder createWithCenterLocation:COORDINATE_AMSTERDAM] withSpeedConsumptionInkWhPairs:consumption count:1]
                             withVehicleWeight:1600]
                            withCurrentChargeInkWh:43]
                           withMaxChargeInkWh:85]
                          withAuxiliaryPowerInkW:1.7]
                         withAccelerationEfficiency:0.33]
                        withDecelerationEfficiency:0.33]
                       withUphillEfficiency:0.33]
                      withDownhillEfficiency:0.33]
                     withVehicleEngineType:TTOptionVehicleEngineTypeElectric]
                    withTimeBudgetInSeconds:7200]
                build];
            break;
            
        default:
            break;
    }
    
    [_reachableRange findReachableRangeWithQuery:query];
}

- (void)resultSupportingPoints:(NSInteger)index {
    CLLocationCoordinate2D exampleCoordinateDest = CLLocationCoordinate2DMake(40.209408, -8.423741);
    CLLocationCoordinate2D exampleCoordinateOrigin = CLLocationCoordinate2DMake(40.10995732392718, -8.501433134078981);
    
    NSMutableArray<NSValue *>* supportingPoints = [[NSMutableArray alloc] init];
    supportingPoints = [self supportingPoints];
    
    switch (index) {
            case 0:
                _query = [[[[[[[TTRouteQueryBuilder createWithDest:(CLLocationCoordinate2D)exampleCoordinateDest andOrig:(CLLocationCoordinate2D)exampleCoordinateOrigin]
                          withMaxAlternatives:1]
                         withMinDeviationTime:0]
                        withSupportingPoints:supportingPoints]
                       withMinDeviationDistance:0]
                          withTraffic:NO]
                      build];
            
            break;
            case 1:
                _query = [[[[[[[TTRouteQueryBuilder createWithDest:(CLLocationCoordinate2D)exampleCoordinateDest andOrig:(CLLocationCoordinate2D)exampleCoordinateOrigin]
                          withMaxAlternatives:1]
                         withMinDeviationTime:0]
                        withSupportingPoints:supportingPoints]
                       withMinDeviationDistance:10000]
                          withTraffic:NO]
                      build];
            break;
        default:
            break;
    }
    [self displayProgressDialog];
    [_routePlanner planRouteWithQuery:_query];
}

- (void)resultAlternativeRoute:(NSInteger)index {
    _polylineIDArray = [NSMutableDictionary new];
    _index = index;
    NSInteger numberOfRoutes = 0;
    switch (_index) {
        case 0:
            numberOfRoutes = 1;
            break;
        case 1:
            numberOfRoutes = 3;
            break;
        case 2:
            numberOfRoutes = 5;
            break;
        default:
            break;
    }
    _query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_ROTTERDAM andOrig:COORDINATE_AMSTERDAM]
              withMaxAlternatives:numberOfRoutes]
              withTraffic:NO]
              build];
    
    [_routePlanner planRouteWithQuery:_query];
    
    
    [self displayProgressDialog];
}

- (void)resultRouteTimeTravel:(NSInteger)index {
    self.index = index;
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"\n\n\n\n\n\n\n\n\n\n\n" message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    NSTimeInterval secondsInEightHours = 5 * 60 * 60; //+ 5 h
    NSDate *pickerInitialDate = [[NSDate date] dateByAddingTimeInterval:secondsInEightHours];
    UIDatePicker *picker = [[UIDatePicker alloc] init];
    [picker setDatePickerMode:UIDatePickerModeDateAndTime];
    [alertController.view addSubview:picker];
    picker.date = pickerInitialDate;
    picker.minuteInterval = 5;

    [alertController addAction:({
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction UNUSED_PARAM *action) {
            NSComparisonResult datesDiff = [[NSDate date] compare:picker.date];
            if(datesDiff == NSOrderedDescending) {
                [self setShowToastEnable:YES];
                [self showToast:@"The departure and arrival can not be in the past" withTimer:YES];
                return;
            }
            NSDate *date = picker.date;
            
            switch (index) {
                case 0:
                    [self.etaView setPanelIconToDefault];
                    self.query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_ROTTERDAM andOrig:COORDINATE_AMSTERDAM]
                               withDepartAt: date]withTraffic:NO]
                              build];
                    break;
                case 1:
                    [self.etaView setPanelIconToArrivalTime];
                    self.query = [[[[TTRouteQueryBuilder createWithDest:COORDINATE_ROTTERDAM andOrig:COORDINATE_AMSTERDAM]
                               withArriveAt: date]
                              withTraffic:NO]
                              build];
                    break;
                default:
                    break;
            }
            
            [self.routePlanner planRouteWithQuery:self->_query];
            
            [self displayProgressDialog];
        }];
        action;
    })];
    UIAlertAction* noButton = [UIAlertAction
                               actionWithTitle:@"No, thanks"
                               style:UIAlertActionStyleDefault
                               handler:^(UIAlertAction * UNUSED_PARAM action) {
                                   [self deselectOptionIndexs:@[@(2), @(1)]];
                               }];
    [alertController addAction:noButton];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)routeClicked:(TTMapRoute * _Nonnull)route {
    for(TTMapRoute *mapRoute in self.mapView.routeManager.routes) {
        mapRoute.active = route == mapRoute;
    }
    [self updateETA:route];
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id <UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> UNUSED_PARAM context) {
    } completion:^(id<UIViewControllerTransitionCoordinatorContext> UNUSED_PARAM context) {
        if(self.mapView.routeManager.routes.count > 0) {
            [self displayRouteOverview];
        }
    }];
}

- (NSMutableArray<NSValue *>*)supportingPoints{
    NSMutableArray<NSValue *>* suppPoint = [[NSMutableArray alloc] init];
    
    CLLocationCoordinate2D suppPoint1 = CLLocationCoordinate2DMake(40.10995732392718, -8.501433134078981);
    NSValue * valuee1 = [[NSValue alloc] initWithBytes: &suppPoint1 objCType: @encode(CLLocationCoordinate2D)];
    [suppPoint addObject:valuee1];
    CLLocationCoordinate2D suppPoint2 = CLLocationCoordinate2DMake(40.11115121590874, -8.500000834465029);
    NSValue * valuee2 = [[NSValue alloc] initWithBytes: &suppPoint2 objCType: @encode(CLLocationCoordinate2D)];
    [suppPoint addObject:valuee2];
    CLLocationCoordinate2D suppPoint3 = CLLocationCoordinate2DMake(40.11089684892725, -8.497683405876161);
    NSValue * valuee3 = [[NSValue alloc] initWithBytes: &suppPoint3 objCType: @encode(CLLocationCoordinate2D)];
    [suppPoint addObject:valuee3];
    CLLocationCoordinate2D suppPoint4 = CLLocationCoordinate2DMake(40.11192251642396, -8.498423695564272);
    NSValue * valuee4 = [[NSValue alloc] initWithBytes: &suppPoint4 objCType: @encode(CLLocationCoordinate2D)];
    [suppPoint addObject:valuee4];
    CLLocationCoordinate2D suppPoint5 = CLLocationCoordinate2DMake(40.209408, -8.423741);
    NSValue * valuee5 = [[NSValue alloc] initWithBytes: &suppPoint5 objCType: @encode(CLLocationCoordinate2D)];
    [suppPoint addObject:valuee5];
    return suppPoint;
}

@end
