/**
 * Copyright (c) 2017 TomTom N.V. All rights reserved.
 *
 * This software is the proprietary copyright of TomTom N.V. and its subsidiaries and may be used
 * for internal evaluation purposes or commercial use strictly subject to separate licensee
 * agreement between you and TomTom. If you are the licensee, you are only permitted to use
 * this Software in accordance with the terms of your license agreement. If you are not the
 * licensee then you are not authorised to use this software in any manner and should
 * immediately return it to TomTom N.V.
 */

#import "SearchViewController.h"
#import "LocationManager.h"
#import <TomTomOnlineUtils/TomTomOnlineUtils.h>

NSString *const kCellIdentifier = @"kCellIdentifier";

@interface SearchViewController () <TTSearchDelegate>

@property (nonatomic, strong) TTSearch *search;
@property (nonatomic) LocationManager *locationManager;
@end

@implementation SearchViewController

- (void)viewDidLoad {
    [super viewDidLoad]; 
    [self.view setAccessibilityIdentifier:@"tableView"];

    self.tableView.rowHeight = UITableViewAutomaticDimension;
    self.tableView.estimatedRowHeight = 100;
    self.tableView.accessibilityLabel = @"SearchTableView";
    self.tableView.accessibilityIdentifier = @"SearchTableView";
    
    self.locationManager = [LocationManager sharedInstance];
    [self.locationManager startService];
    
    UIImage *menuImage = [[UIImage imageNamed:@"BtnPixelDot"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
    UIBarButtonItem *optionsItem = [[UIBarButtonItem alloc] initWithImage:menuImage
                                                                    style:UIBarButtonItemStylePlain
                                                                   target:self.navigationController
                                                                   action:@selector(popViewControllerAnimated:)];
    self.navigationItem.leftBarButtonItem = optionsItem;
    
    [self initSearchEngine];
}

- (BOOL)prefersStatusBarHidden {
    return NO;
}

- (void)performAddressSearchWithQuery:(NSString *)query fuzzyMin:(int)min fuzzyMax:(int)max position:(CLLocationCoordinate2D)position{
    TTSearchQueryBuilder *queryBuilder = [[[[TTSearchQueryBuilder
                                            createWithTerm:query]
                                             withMaxFuzzyLevel:@(max)]
                                             withPosition:position]
                                            withMinFuzzyLevel:@(min)];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}

- (void)initSearchEngine {
    TTSearch *search = [[TTSearch alloc] init];
    search.delegate = self;
    self.search = search;
}

- (void)performAddressSearchWithQuery:(NSString *)query {
    TTSearchQueryBuilder *queryBuilder = [TTSearchQueryBuilder
                                            createWithTerm:query];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}

- (void)performAddressSearchWithQuery:(NSString *)query
                             position:(CLLocationCoordinate2D)position {
    TTSearchQueryBuilder *queryBuilder = [TTSearchQueryBuilder
                                            createWithTerm:query];
    [queryBuilder withPosition:(CLLocationCoordinate2D)position withRadius:30000];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}

- (void)performAddressSearchWithQuery:(NSString *)query
                             language:(TTLanguage)language {
    TTSearchQueryBuilder *queryBuilder = [TTSearchQueryBuilder
                                            createWithTerm:query];
    [queryBuilder withLanguage:language];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}


- (void)performCategorySearchWithQuery:(NSString *)query {
    TTSearchQueryBuilder *queryBuilder = [TTSearchQueryBuilder
                                            createWithTerm:query];
    [queryBuilder withCategory:YES];
    
    [queryBuilder withPosition:self.locationManager.location.coordinate];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}

- (void)performAutocompleteSearchWithQuery:(NSString *)query {
    TTSearchQueryBuilder *queryBuilder = [[TTSearchQueryBuilder
                                             createWithTerm:query]
                                            withTypeAhead:true];
    TTSearchQuery *searchQuery = [queryBuilder build];
    [self.search searchWithQuery:searchQuery];
}

- (void)cancelSearch {
    [self.search cancel];
}

#pragma mark - TTSearchDelegate

-(void)search:(TTSearch *)search
completedWithResponse:(TTSearchResponse *)response {
    self.items = response.results;
    [self.tableView reloadData];
}

-(void)search:(TTSearch*)search
failedWithError:(TTResponseError *)error {
    self.items = @[[error.userInfo objectForKey:@"description"]];
    [self.tableView reloadData];
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UNUSED_PARAM UITableView *)tableView numberOfRowsInSection:(UNUSED_PARAM NSInteger)section {
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    return (NSInteger)self.items.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kCellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:kCellIdentifier];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }

    if ([self.items[(NSUInteger)indexPath.row] isKindOfClass:TTSearchResult.class]) {
        TTSearchResult *item = self.items[(NSUInteger)indexPath.row];
        [self configureCell:cell forItemObjectSearch:item];
    }
    return cell;
}

- (void)configureCell:(UITableViewCell *)cell forItemObjectSearch:(TTSearchResult *)item {
    cell.textLabel.numberOfLines = 4;
    cell.textLabel.text = [SearchViewController createDescription:item showDistance:YES];
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UNUSED_PARAM UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
}

- (void)scrollViewDidScroll:(UNUSED_PARAM UIScrollView *)scrollView {
    [self.searchBar resignFirstResponder];
}

#pragma mark - Static

+(NSString *)createDescription:(TTSearchResult *)result showDistance:(BOOL)showDistance {
    if(showDistance) {
        CLLocationDistance distance = [[LocationManager sharedInstance] distanceFromCurrentLocation:result.position];
        return [NSString stringWithFormat:@"(%.1f km) %@ %@ %@", distance / 1000.0, result.address.country, result.address.freeformAddress, result.address.countryCode];
    } else {
        return [NSString stringWithFormat:@"%@ %@ %@", result.address.country, result.address.freeformAddress, result.address.countryCode];
    }
}

@end
